@@ -9,7 +9,15 @@ has 'lexical_environment' => (
   is => 'rw',
   required => 1,
   lazy => 1,
-  default => sub { Lexical::Persistence->new }
+  default => sub { undef }
+);
+
+has 'frame' => (
+  isa => 'Int',
+  is => 'rw',
+  required => 1,
+  lazy => 1,
+  default => sub { 0 },
 );
 
 around 'mangle_line' => sub {
@@ -17,18 +25,48 @@ around 'mangle_line' => sub {
   my ($self, @rest) = @_;
   my $line = $self->$orig(@rest);
   my $lp = $self->lexical_environment;
+  my $frame_delta;
+  my $frame = $self->frame;
 
-  # exactly the same as LexEnv except for this call
-  $lp->set_context(
-    _ => {
-        %Carp::REPL::environment, # XXX gross!
-        %{$lp->get_context('_')},
-    });
+  if (!defined($lp))
+  {
+    $lp = $self->lexical_environment(Lexical::Persistence->new);
+    $frame_delta = 0;
+  }
+
+  if ($line =~ /^:up?$/i)
+  {
+    if ($frame + 1 == @Carp::REPL::environments)
+    {
+      return q{"You're already at the top frame."};
+    }
+    $frame_delta = 1;
+  }
+  elsif ($line =~ /^:d(?:own)?$/i)
+  {
+    if ($frame == 0)
+    {
+      return q{"You're already at the bottom frame."};
+    }
+    $frame_delta = -1;
+  }
+
+  if (defined($frame_delta))
+  {
+    $frame += $frame_delta;
+    $self->frame($frame);
+    $lp->set_context(
+        _ => {
+            %{$Carp::REPL::environments[$frame]},
+        });
+    my ($package, $file, $line) = @{$Carp::REPL::packages[$frame]};
+    return qq{"Now at $file:$line (frame $frame)."} if $frame_delta != 0;
+  }
 
   # Collate my declarations for all LP context vars then add '';
   # so an empty statement doesn't return anything (with a no warnings
   # to prevent "Useless use ..." warning)
-  return "package $Carp::REPL::package;\n"
+  return "package $Carp::REPL::packages[$frame][0];\n"
          .join('', map { "my $_;\n" } keys %{$lp->get_context('_')})
          .qq{{ no warnings 'void'; ''; }\n}.$line;
 };
@@ -46,17 +84,20 @@ Devel::REPL::Plugin::LexEnvCarp - Devel::REPL plugin for Carp::REPL
 
 =head1 VERSION
 
-Version 0.02
+Version 0.03
 
 =cut
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 =head1 SYNOPSIS
 
 This implements exactly the same thing as the Devel::REPL::Plugin::LexEnv
 module except the lexical environment exposed by Carp::REPL is mixed in.
 
+It also adds a few extra commands like :up and :down to move up and down the
+stack.
+
 =head1 AUTHOR
 
 Shawn M Moore, C<< <sartak at gmail.com> >>
